@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,12 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import errno
 import os
 import subprocess
 import sys
 import tempfile
-import urllib
 
 import bzrlib
 from bzrlib import (
@@ -89,7 +90,7 @@ class MailClient(object):
 
 
 class Editor(MailClient):
-    """DIY mail client that uses commit message editor"""
+    __doc__ = """DIY mail client that uses commit message editor"""
 
     supports_body = True
 
@@ -230,13 +231,13 @@ class BodyExternalMailClient(MailClient):
 
 
 class ExternalMailClient(BodyExternalMailClient):
-    """An external mail client."""
+    __doc__ = """An external mail client."""
 
     supports_body = False
 
 
 class Evolution(BodyExternalMailClient):
-    """Evolution mail client."""
+    __doc__ = """Evolution mail client."""
 
     _client_commands = ['evolution']
 
@@ -258,7 +259,7 @@ mail_client_registry.register('evolution', Evolution,
 
 
 class Mutt(BodyExternalMailClient):
-    """Mutt mail client."""
+    __doc__ = """Mutt mail client."""
 
     _client_commands = ['mutt']
 
@@ -286,7 +287,7 @@ mail_client_registry.register('mutt', Mutt,
 
 
 class Thunderbird(BodyExternalMailClient):
-    """Mozilla Thunderbird (or Icedove)
+    __doc__ = """Mozilla Thunderbird (or Icedove)
 
     Note that Thunderbird 1.5 is buggy and does not support setting
     "to" simultaneously with including a attachment.
@@ -310,7 +311,7 @@ class Thunderbird(BodyExternalMailClient):
             message_options['attachment'] = urlutils.local_path_to_url(
                 attach_path)
         if body is not None:
-            options_list = ['body=%s' % urllib.quote(self._encode_safe(body))]
+            options_list = ['body=%s' % urlutils.quote(self._encode_safe(body))]
         else:
             options_list = []
         options_list.extend(["%s='%s'" % (k, v) for k, v in
@@ -321,7 +322,7 @@ mail_client_registry.register('thunderbird', Thunderbird,
 
 
 class KMail(ExternalMailClient):
-    """KDE mail client."""
+    __doc__ = """KDE mail client."""
 
     _client_commands = ['kmail']
 
@@ -341,7 +342,7 @@ mail_client_registry.register('kmail', KMail,
 
 
 class Claws(ExternalMailClient):
-    """Claws mail client."""
+    __doc__ = """Claws mail client."""
 
     supports_body = True
 
@@ -352,15 +353,15 @@ class Claws(ExternalMailClient):
         """See ExternalMailClient._get_compose_commandline"""
         compose_url = []
         if from_ is not None:
-            compose_url.append('from=' + urllib.quote(from_))
+            compose_url.append('from=' + urlutils.quote(from_))
         if subject is not None:
-            # Don't use urllib.quote_plus because Claws doesn't seem
+            # Don't use urlutils.quote_plus because Claws doesn't seem
             # to recognise spaces encoded as "+".
             compose_url.append(
-                'subject=' + urllib.quote(self._encode_safe(subject)))
+                'subject=' + urlutils.quote(self._encode_safe(subject)))
         if body is not None:
             compose_url.append(
-                'body=' + urllib.quote(self._encode_safe(body)))
+                'body=' + urlutils.quote(self._encode_safe(body)))
         # to must be supplied for the claws-mail --compose syntax to work.
         if to is None:
             raise errors.NoMailAddressSpecified()
@@ -387,7 +388,7 @@ mail_client_registry.register('claws', Claws,
 
 
 class XDGEmail(BodyExternalMailClient):
-    """xdg-email attempts to invoke the user's preferred mail client"""
+    __doc__ = """xdg-email attempts to invoke the user's preferred mail client"""
 
     _client_commands = ['xdg-email']
 
@@ -409,7 +410,7 @@ mail_client_registry.register('xdg-email', XDGEmail,
 
 
 class EmacsMail(ExternalMailClient):
-    """Call emacsclient to have a mail buffer.
+    __doc__ = """Call emacsclient to have a mail buffer.
 
     This only work for emacs >= 22.1 due to recent -e/--eval support.
 
@@ -519,7 +520,7 @@ mail_client_registry.register('emacsclient', EmacsMail,
 
 
 class MAPIClient(BodyExternalMailClient):
-    """Default Windows mail client launched using MAPI."""
+    __doc__ = """Default Windows mail client launched using MAPI."""
 
     def _compose(self, prompt, to, subject, attach_path, mime_subtype,
                  extension, body=None):
@@ -540,7 +541,7 @@ mail_client_registry.register('mapi', MAPIClient,
 
 
 class MailApp(BodyExternalMailClient):
-    """Use MacOS X's Mail.app for sending email messages.
+    __doc__ = """Use MacOS X's Mail.app for sending email messages.
 
     Although it would be nice to use appscript, it's not installed
     with the shipped Python installations.  We instead build an
@@ -599,7 +600,7 @@ mail_client_registry.register('mail.app', MailApp,
 
 
 class DefaultMail(MailClient):
-    """Default mail handling.  Tries XDGEmail (or MAPIClient on Windows),
+    __doc__ = """Default mail handling.  Tries XDGEmail (or MAPIClient on Windows),
     falls back to Editor"""
 
     supports_body = True
