@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import errno
 import os
 import re
@@ -26,8 +28,10 @@ from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from datetime import datetime
 import getpass
+import locale
 import ntpath
 import posixpath
+import select
 # We need to import both shutil and rmtree as we export the later on posix
 # and need the former on windows
 import shutil
@@ -42,15 +46,20 @@ import unicodedata
 
 from bzrlib import (
     cache_utf8,
+    config,
     errors,
     trace,
     win32utils,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
     deprecated_function,
     deprecated_in,
+    deprecated_passed,
+    warn as warn_deprecated,
     )
 
 from hashlib import (
@@ -60,7 +69,7 @@ from hashlib import (
 
 
 import bzrlib
-from bzrlib import symbol_versioning
+from bzrlib import symbol_versioning, _fs_enc
 
 
 # Cross platform wall-clock time functionality with decent resolution.
@@ -88,8 +97,8 @@ def get_unicode_argv():
         user_encoding = get_user_encoding()
         return [a.decode(user_encoding) for a in sys.argv[1:]]
     except UnicodeDecodeError:
-        raise errors.BzrError("Parameter %r encoding is unsupported by %s "
-            "application locale." % (a, user_encoding))
+        raise errors.BzrError(gettext("Parameter {0!r} encoding is unsupported by {1} "
+            "application locale.").format(a, user_encoding))
 
 
 def make_readonly(filename):
@@ -97,14 +106,33 @@ def make_readonly(filename):
     mod = os.lstat(filename).st_mode
     if not stat.S_ISLNK(mod):
         mod = mod & 0777555
-        os.chmod(filename, mod)
+        chmod_if_possible(filename, mod)
 
 
 def make_writable(filename):
     mod = os.lstat(filename).st_mode
     if not stat.S_ISLNK(mod):
         mod = mod | 0200
-        os.chmod(filename, mod)
+        chmod_if_possible(filename, mod)
+
+
+def chmod_if_possible(filename, mode):
+    # Set file mode if that can be safely done.
+    # Sometimes even on unix the filesystem won't allow it - see
+    # https://bugs.launchpad.net/bzr/+bug/606537
+    try:
+        # It is probably faster to just do the chmod, rather than
+        # doing a stat, and then trying to compare
+        os.chmod(filename, mode)
+    except (IOError, OSError),e:
+        # Permission/access denied seems to commonly happen on smbfs; there's
+        # probably no point warning about it.
+        # <https://bugs.launchpad.net/bzr/+bug/606537>
+        if getattr(e, 'errno') in (errno.EPERM, errno.EACCES):
+            trace.mutter("ignore error on chmod of %r: %r" % (
+                filename, e))
+            return
+        raise
 
 
 def minimum_path_selection(paths):
@@ -189,7 +217,7 @@ if lexists is None:
             if e.errno == errno.ENOENT:
                 return False;
             else:
-                raise errors.BzrError("lstat/stat of (%r): %r" % (f, e))
+                raise errors.BzrError(gettext("lstat/stat of ({0!r}): {1!r}").format(f, e))
 
 
 def fancy_rename(old, new, rename_func, unlink_func):
@@ -271,19 +299,69 @@ def fancy_rename(old, new, rename_func, unlink_func):
 # choke on a Unicode string containing a relative path if
 # os.getcwd() returns a non-sys.getdefaultencoding()-encoded
 # string.
-_fs_enc = sys.getfilesystemencoding() or 'utf-8'
 def _posix_abspath(path):
     # jam 20060426 rather than encoding to fsencoding
     # copy posixpath.abspath, but use os.getcwdu instead
     if not posixpath.isabs(path):
         path = posixpath.join(getcwd(), path)
-    return posixpath.normpath(path)
+    return _posix_normpath(path)
 
 
 def _posix_realpath(path):
     return posixpath.realpath(path.encode(_fs_enc)).decode(_fs_enc)
 
 
+def _posix_normpath(path):
+    path = posixpath.normpath(path)
+    # Bug 861008: posixpath.normpath() returns a path normalized according to
+    # the POSIX standard, which stipulates (for compatibility reasons) that two
+    # leading slashes must not be simplified to one, and only if there are 3 or
+    # more should they be simplified as one. So we treat the leading 2 slashes
+    # as a special case here by simply removing the first slash, as we consider
+    # that breaking POSIX compatibility for this obscure feature is acceptable.
+    # This is not a paranoid precaution, as we notably get paths like this when
+    # the repo is hosted at the root of the filesystem, i.e. in "/".    
+    if path.startswith('//'):
+        path = path[1:]
+    return path
+
+
+def _posix_path_from_environ(key):
+    """Get unicode path from `key` in environment or None if not present
+
+    Note that posix systems use arbitrary byte strings for filesystem objects,
+    so a path that raises BadFilenameEncoding here may still be accessible.
+    """
+    val = os.environ.get(key, None)
+    if val is None:
+        return val
+    try:
+        return val.decode(_fs_enc)
+    except UnicodeDecodeError:
+        # GZ 2011-12-12:Ideally want to include `key` in the exception message
+        raise errors.BadFilenameEncoding(val, _fs_enc)
+
+
+def _posix_get_home_dir():
+    """Get the home directory of the current user as a unicode path"""
+    path = posixpath.expanduser("~")
+    try:
+        return path.decode(_fs_enc)
+    except UnicodeDecodeError:
+        raise errors.BadFilenameEncoding(path, _fs_enc)
+
+
+def _posix_getuser_unicode():
+    """Get username from environment or password database as unicode"""
+    name = getpass.getuser()
+    user_encoding = get_user_encoding()
+    try:
+        return name.decode(user_encoding)
+    except UnicodeDecodeError:
+        raise errors.BzrError("Encoding of username %r is unsupported by %s "
+            "application locale." % (name, user_encoding))
+
+
 def _win32_fixdrive(path):
     """Force drive letters to be consistent.
 
@@ -377,7 +455,10 @@ def _mac_getcwd():
 abspath = _posix_abspath
 realpath = _posix_realpath
 pathjoin = os.path.join
-normpath = os.path.normpath
+normpath = _posix_normpath
+path_from_environ = _posix_path_from_environ
+_get_home_dir = _posix_get_home_dir
+getuser_unicode = _posix_getuser_unicode
 getcwd = os.getcwdu
 rename = os.rename
 dirname = os.path.dirname
@@ -439,6 +520,9 @@ if sys.platform == 'win32':
     f = win32utils.get_unicode_argv     # special function or None
     if f is not None:
         get_unicode_argv = f
+    path_from_environ = win32utils.get_environ_unicode
+    _get_home_dir = win32utils.get_home_location
+    getuser_unicode = win32utils.get_user_name
 
 elif sys.platform == 'darwin':
     getcwd = _mac_getcwd
@@ -876,19 +960,16 @@ def filesize(f):
     return os.fstat(f.fileno())[stat.ST_SIZE]
 
 
-# Define rand_bytes based on platform.
-try:
-    # Python 2.4 and later have os.urandom,
-    # but it doesn't work on some arches
-    os.urandom(1)
-    rand_bytes = os.urandom
-except (NotImplementedError, AttributeError):
-    # If python doesn't have os.urandom, or it doesn't work,
-    # then try to first pull random data from /dev/urandom
+# Alias os.urandom to support platforms (which?) without /dev/urandom and 
+# override if it doesn't work. Avoid checking on windows where there is
+# significant initialisation cost that can be avoided for some bzr calls.
+
+rand_bytes = os.urandom
+
+if rand_bytes.__module__ != "nt":
     try:
-        rand_bytes = file('/dev/urandom', 'rb').read
-    # Otherwise, use this hack as a last resort
-    except (IOError, OSError):
+        rand_bytes(1)
+    except NotImplementedError:
         # not well seeded, but better than nothing
         def rand_bytes(n):
             import random
@@ -924,7 +1005,7 @@ def splitpath(p):
     rps = []
     for f in ps:
         if f == '..':
-            raise errors.BzrError("sorry, %r not allowed in path" % f)
+            raise errors.BzrError(gettext("sorry, %r not allowed in path") % f)
         elif (f == '.') or (f == ''):
             pass
         else:
@@ -935,7 +1016,7 @@ def splitpath(p):
 def joinpath(p):
     for f in p:
         if (f == '..') or (f is None) or (f == ''):
-            raise errors.BzrError("sorry, %r not allowed in path" % f)
+            raise errors.BzrError(gettext("sorry, %r not allowed in path") % f)
     return pathjoin(*p)
 
 
@@ -985,8 +1066,7 @@ def failed_to_load_extension(exception):
 def report_extension_load_failures():
     if not _extension_load_failures:
         return
-    from bzrlib.config import GlobalConfig
-    if GlobalConfig().get_user_option_as_bool('ignore_missing_extensions'):
+    if config.GlobalStack().get('ignore_missing_extensions'):
         return
     # the warnings framework should by default show this only once
     from bzrlib.trace import warning
@@ -1154,7 +1234,7 @@ def relpath(base, path):
 
     if len(base) < MIN_ABS_PATHLENGTH:
         # must have space for e.g. a drive letter
-        raise ValueError('%r is too short to calculate a relative path'
+        raise ValueError(gettext('%r is too short to calculate a relative path')
             % (base,))
 
     rp = abspath(path)
@@ -1735,7 +1815,6 @@ def _walkdirs_utf8(top, prefix=""):
     """
     global _selected_dir_reader
     if _selected_dir_reader is None:
-        fs_encoding = _fs_enc.upper()
         if sys.platform == "win32" and win32utils.winver == 'Windows NT':
             # Win98 doesn't have unicode apis like FindFirstFileW
             # TODO: We possibly could support Win98 by falling back to the
@@ -1747,8 +1826,7 @@ def _walkdirs_utf8(top, prefix=""):
                 _selected_dir_reader = Win32ReadDir()
             except ImportError:
                 pass
-        elif fs_encoding in ('UTF-8', 'US-ASCII', 'ANSI_X3.4-1968'):
-            # ANSI_X3.4-1968 is a form of ASCII
+        elif _fs_enc in ('utf-8', 'ascii'):
             try:
                 from bzrlib._readdir_pyx import UTF8DirReader
                 _selected_dir_reader = UTF8DirReader()
@@ -1913,76 +1991,53 @@ def compare_paths_prefix_order(path_a, path_b):
 _cached_user_encoding = None
 
 
-def get_user_encoding(use_cache=True):
+def get_user_encoding(use_cache=DEPRECATED_PARAMETER):
     """Find out what the preferred user encoding is.
 
     This is generally the encoding that is used for command line parameters
     and file contents. This may be different from the terminal encoding
     or the filesystem encoding.
 
-    :param  use_cache:  Enable cache for detected encoding.
-                        (This parameter is turned on by default,
-                        and required only for selftesting)
-
     :return: A string defining the preferred user encoding
     """
     global _cached_user_encoding
-    if _cached_user_encoding is not None and use_cache:
+    if deprecated_passed(use_cache):
+        warn_deprecated("use_cache should only have been used for tests",
+            DeprecationWarning, stacklevel=2) 
+    if _cached_user_encoding is not None:
         return _cached_user_encoding
 
-    if sys.platform == 'darwin':
-        # python locale.getpreferredencoding() always return
-        # 'mac-roman' on darwin. That's a lie.
-        sys.platform = 'posix'
-        try:
-            if os.environ.get('LANG', None) is None:
-                # If LANG is not set, we end up with 'ascii', which is bad
-                # ('mac-roman' is more than ascii), so we set a default which
-                # will give us UTF-8 (which appears to work in all cases on
-                # OSX). Users are still free to override LANG of course, as
-                # long as it give us something meaningful. This work-around
-                # *may* not be needed with python 3k and/or OSX 10.5, but will
-                # work with them too -- vila 20080908
-                os.environ['LANG'] = 'en_US.UTF-8'
-            import locale
-        finally:
-            sys.platform = 'darwin'
+    if os.name == 'posix' and getattr(locale, 'CODESET', None) is not None:
+        # Use the existing locale settings and call nl_langinfo directly
+        # rather than going through getpreferredencoding. This avoids
+        # <http://bugs.python.org/issue6202> on OSX Python 2.6 and the
+        # possibility of the setlocale call throwing an error.
+        user_encoding = locale.nl_langinfo(locale.CODESET)
     else:
-        import locale
+        # GZ 2011-12-19: On windows could call GetACP directly instead.
+        user_encoding = locale.getpreferredencoding(False)
 
     try:
-        user_encoding = locale.getpreferredencoding()
-    except locale.Error, e:
-        sys.stderr.write('bzr: warning: %s\n'
-                         '  Could not determine what text encoding to use.\n'
-                         '  This error usually means your Python interpreter\n'
-                         '  doesn\'t support the locale set by $LANG (%s)\n'
-                         "  Continuing with ascii encoding.\n"
-                         % (e, os.environ.get('LANG')))
-        user_encoding = 'ascii'
-
-    # Windows returns 'cp0' to indicate there is no code page. So we'll just
-    # treat that as ASCII, and not support printing unicode characters to the
-    # console.
-    #
-    # For python scripts run under vim, we get '', so also treat that as ASCII
-    if user_encoding in (None, 'cp0', ''):
-        user_encoding = 'ascii'
-    else:
-        # check encoding
-        try:
-            codecs.lookup(user_encoding)
-        except LookupError:
+        user_encoding = codecs.lookup(user_encoding).name
+    except LookupError:
+        if user_encoding not in ("", "cp0"):
             sys.stderr.write('bzr: warning:'
                              ' unknown encoding %s.'
                              ' Continuing with ascii encoding.\n'
                              % user_encoding
                             )
-            user_encoding = 'ascii'
-
-    if use_cache:
-        _cached_user_encoding = user_encoding
+        user_encoding = 'ascii'
+    else:
+        # Get 'ascii' when setlocale has not been called or LANG=C or unset.
+        if user_encoding == 'ascii':
+            if sys.platform == 'darwin':
+                # OSX is special-cased in Python to have a UTF-8 filesystem
+                # encoding and previously had LANG set here if not present.
+                user_encoding = 'utf-8'
+            # GZ 2011-12-19: Maybe UTF-8 should be the default in this case
+            #                for some other posix platforms as well.
 
+    _cached_user_encoding = user_encoding
     return user_encoding
 
 
@@ -1997,7 +2052,6 @@ def get_host_name():
     behaves inconsistently on different platforms.
     """
     if sys.platform == "win32":
-        import win32utils
         return win32utils.get_host_name()
     else:
         import socket
@@ -2178,15 +2232,18 @@ def file_kind_from_stat_mode_thunk(mode):
     return file_kind_from_stat_mode(mode)
 file_kind_from_stat_mode = file_kind_from_stat_mode_thunk
 
-
-def file_kind(f, _lstat=os.lstat):
+def file_stat(f, _lstat=os.lstat):
     try:
-        return file_kind_from_stat_mode(_lstat(f).st_mode)
+        # XXX cache?
+        return _lstat(f)
     except OSError, e:
         if getattr(e, 'errno', None) in (errno.ENOENT, errno.ENOTDIR):
             raise errors.NoSuchFile(f)
         raise
 
+def file_kind(f, _lstat=os.lstat):
+    stat_value = file_stat(f, _lstat)
+    return file_kind_from_stat_mode(stat_value.st_mode)
 
 def until_no_eintr(f, *a, **kw):
     """Run f(*a, **kw), retrying if an EINTR error occurs.
@@ -2236,13 +2293,13 @@ def re_compile_checked(re_string, flags=0, where=""):
 
 
 if sys.platform == "win32":
-    import msvcrt
     def getchar():
+        import msvcrt
         return msvcrt.getch()
 else:
-    import tty
-    import termios
     def getchar():
+        import tty
+        import termios
         fd = sys.stdin.fileno()
         settings = termios.tcgetattr(fd)
         try:
@@ -2252,7 +2309,7 @@ else:
             termios.tcsetattr(fd, termios.TCSADRAIN, settings)
         return ch
 
-if sys.platform == 'linux2':
+if sys.platform.startswith('linux'):
     def _local_concurrency():
         try:
             return os.sysconf('SC_NPROCESSORS_ONLN')
@@ -2297,14 +2354,15 @@ def local_concurrency(use_cache=True):
     if concurrency is None:
         try:
             import multiprocessing
-        except ImportError:
+            concurrency = multiprocessing.cpu_count()
+        except (ImportError, NotImplementedError):
             # multiprocessing is only available on Python >= 2.6
+            # and multiprocessing.cpu_count() isn't implemented on all
+            # platforms
             try:
                 concurrency = _local_concurrency()
             except (OSError, IOError):
                 pass
-        else:
-            concurrency = multiprocessing.cpu_count()
     try:
         concurrency = int(concurrency)
     except (TypeError, ValueError):
@@ -2372,30 +2430,6 @@ else:
     open_file = open
 
 
-def getuser_unicode():
-    """Return the username as unicode.
-    """
-    try:
-        user_encoding = get_user_encoding()
-        username = getpass.getuser().decode(user_encoding)
-    except UnicodeDecodeError:
-        raise errors.BzrError("Can't decode username as %s." % \
-                user_encoding)
-    except ImportError, e:
-        if sys.platform != 'win32':
-            raise
-        if str(e) != 'No module named pwd':
-            raise
-        # https://bugs.launchpad.net/bzr/+bug/660174
-        # getpass.getuser() is unable to return username on Windows
-        # if there is no USERNAME environment variable set.
-        # That could be true if bzr is running as a service,
-        # e.g. running `bzr serve` as a service on Windows.
-        # We should not fail with traceback in this case.
-        username = u'UNKNOWN'
-    return username
-
-
 def available_backup_name(base, exists):
     """Find a non-existing backup file name.
 
@@ -2498,3 +2532,31 @@ def fdatasync(fileno):
     fn = getattr(os, 'fdatasync', getattr(os, 'fsync', None))
     if fn is not None:
         fn(fileno)
+
+
+def ensure_empty_directory_exists(path, exception_class):
+    """Make sure a local directory exists and is empty.
+    
+    If it does not exist, it is created.  If it exists and is not empty, an
+    instance of exception_class is raised.
+    """
+    try:
+        os.mkdir(path)
+    except OSError, e:
+        if e.errno != errno.EEXIST:
+            raise
+        if os.listdir(path) != []:
+            raise exception_class(path)
+
+
+def is_environment_error(evalue):
+    """True if exception instance is due to a process environment issue
+
+    This includes OSError and IOError, but also other errors that come from
+    the operating system or core libraries but are not subclasses of those.
+    """
+    if isinstance(evalue, (EnvironmentError, select.error)):
+        return True
+    if sys.platform == "win32" and win32utils._is_pywintypes_error(evalue):
+        return True
+    return False
