@@ -16,6 +16,8 @@
 
 """Repository formats using CHK inventories and groupcompress compression."""
 
+from __future__ import absolute_import
+
 import time
 
 from bzrlib import (
@@ -26,7 +28,6 @@ from bzrlib import (
     errors,
     index as _mod_index,
     inventory,
-    knit,
     osutils,
     pack,
     revision as _mod_revision,
@@ -44,16 +45,19 @@ from bzrlib.groupcompress import (
     GroupCompressVersionedFiles,
     )
 from bzrlib.repofmt.pack_repo import (
+    _DirectPackAccess,
     Pack,
     NewPack,
-    KnitPackRepository,
-    KnitPackStreamSource,
+    PackRepository,
     PackRootCommitBuilder,
     RepositoryPackCollection,
     RepositoryFormatPack,
     ResumedPack,
     Packer,
     )
+from bzrlib.vf_repository import (
+    StreamSource,
+    )
 from bzrlib.static_tuple import StaticTuple
 
 
@@ -352,8 +356,8 @@ class GCCHKPacker(Packer):
         """Build a VersionedFiles instance on top of this group of packs."""
         index_name = index_name + '_index'
         index_to_pack = {}
-        access = knit._DirectPackAccess(index_to_pack,
-                                        reload_func=self._reload_func)
+        access = _DirectPackAccess(index_to_pack,
+                                   reload_func=self._reload_func)
         if for_write:
             # Use new_pack
             if self.new_pack is None:
@@ -603,10 +607,10 @@ class GCCHKCanonicalizingPacker(GCCHKPacker):
     def __init__(self, *args, **kwargs):
         super(GCCHKCanonicalizingPacker, self).__init__(*args, **kwargs)
         self._data_changed = False
-    
+
     def _exhaust_stream(self, source_vf, keys, message, vf_to_stream, pb_offset):
         """Create and exhaust a stream, but don't insert it.
-        
+
         This is useful to get the side-effects of generating a stream.
         """
         self.pb.update('scanning %s' % (message,), pb_offset)
@@ -701,6 +705,8 @@ class GCRepositoryPackCollection(RepositoryPackCollection):
 
     pack_factory = GCPack
     resumed_pack_factory = ResumedGCPack
+    normal_packer_class = GCCHKPacker
+    optimising_packer_class = GCCHKPacker
 
     def _check_new_inventories(self):
         """Detect missing inventories or chk root entries for the new revisions
@@ -788,56 +794,15 @@ class GCRepositoryPackCollection(RepositoryPackCollection):
                 % (sorted(missing_text_keys),))
         return problems
 
-    def _execute_pack_operations(self, pack_operations,
-                                 _packer_class=GCCHKPacker,
-                                 reload_func=None):
-        """Execute a series of pack operations.
 
-        :param pack_operations: A list of [revision_count, packs_to_combine].
-        :param _packer_class: The class of packer to use (default: Packer).
-        :return: None.
-        """
-        # XXX: Copied across from RepositoryPackCollection simply because we
-        #      want to override the _packer_class ... :(
-        for revision_count, packs in pack_operations:
-            # we may have no-ops from the setup logic
-            if len(packs) == 0:
-                continue
-            packer = GCCHKPacker(self, packs, '.autopack',
-                                 reload_func=reload_func)
-            try:
-                result = packer.pack()
-            except errors.RetryWithNewPacks:
-                # An exception is propagating out of this context, make sure
-                # this packer has cleaned up. Packer() doesn't set its new_pack
-                # state into the RepositoryPackCollection object, so we only
-                # have access to it directly here.
-                if packer.new_pack is not None:
-                    packer.new_pack.abort()
-                raise
-            if result is None:
-                return
-            for pack in packs:
-                self._remove_pack_from_memory(pack)
-        # record the newly available packs and stop advertising the old
-        # packs
-        to_be_obsoleted = []
-        for _, packs in pack_operations:
-            to_be_obsoleted.extend(packs)
-        result = self._save_pack_names(clear_obsolete_packs=True,
-                                       obsolete_packs=to_be_obsoleted)
-        return result
-
-
-class CHKInventoryRepository(KnitPackRepository):
-    """subclass of KnitPackRepository that uses CHK based inventories."""
+class CHKInventoryRepository(PackRepository):
+    """subclass of PackRepository that uses CHK based inventories."""
 
     def __init__(self, _format, a_bzrdir, control_files, _commit_builder_class,
         _serializer):
         """Overridden to change pack collection class."""
-        KnitPackRepository.__init__(self, _format, a_bzrdir, control_files,
-            _commit_builder_class, _serializer)
-        # and now replace everything it did :)
+        super(CHKInventoryRepository, self).__init__(_format, a_bzrdir,
+            control_files, _commit_builder_class, _serializer)
         index_transport = self._transport.clone('indices')
         self._pack_collection = GCRepositoryPackCollection(self,
             self._transport, index_transport,
@@ -1015,20 +980,24 @@ class CHKInventoryRepository(KnitPackRepository):
             if record.storage_kind != 'absent':
                 texts[record.key] = record.get_bytes_as('fulltext')
             else:
-                raise errors.NoSuchRevision(self, record.key)
+                texts[record.key] = None
         for key in keys:
-            yield inventory.CHKInventory.deserialise(self.chk_bytes, texts[key], key)
+            bytes = texts[key]
+            if bytes is None:
+                yield (None, key[-1])
+            else:
+                yield (inventory.CHKInventory.deserialise(
+                    self.chk_bytes, bytes, key), key[-1])
 
-    def _iter_inventory_xmls(self, revision_ids, ordering):
+    def _get_inventory_xml(self, revision_id):
+        """Get serialized inventory as a string."""
         # Without a native 'xml' inventory, this method doesn't make sense.
         # However older working trees, and older bundles want it - so we supply
         # it allowing _get_inventory_xml to work. Bundles currently use the
         # serializer directly; this also isn't ideal, but there isn't an xml
-        # iteration interface offered at all for repositories. We could make
-        # _iter_inventory_xmls be part of the contract, even if kept private.
-        inv_to_str = self._serializer.write_inventory_to_string
-        for inv in self.iter_inventories(revision_ids, ordering=ordering):
-            yield inv_to_str(inv), inv.revision_id
+        # iteration interface offered at all for repositories.
+        return self._serializer.write_inventory_to_string(
+            self.get_inventory(revision_id))
 
     def _find_present_inventory_keys(self, revision_keys):
         parent_map = self.inventories.get_parent_map(revision_keys)
@@ -1146,8 +1115,39 @@ class CHKInventoryRepository(KnitPackRepository):
             return GroupCHKStreamSource(self, to_format)
         return super(CHKInventoryRepository, self)._get_source(to_format)
 
+    def _find_inconsistent_revision_parents(self, revisions_iterator=None):
+        """Find revisions with different parent lists in the revision object
+        and in the index graph.
 
-class GroupCHKStreamSource(KnitPackStreamSource):
+        :param revisions_iterator: None, or an iterator of (revid,
+            Revision-or-None). This iterator controls the revisions checked.
+        :returns: an iterator yielding tuples of (revison-id, parents-in-index,
+            parents-in-revision).
+        """
+        if not self.is_locked():
+            raise AssertionError()
+        vf = self.revisions
+        if revisions_iterator is None:
+            revisions_iterator = self._iter_revisions(None)
+        for revid, revision in revisions_iterator:
+            if revision is None:
+                pass
+            parent_map = vf.get_parent_map([(revid,)])
+            parents_according_to_index = tuple(parent[-1] for parent in
+                parent_map[(revid,)])
+            parents_according_to_revision = tuple(revision.parent_ids)
+            if parents_according_to_index != parents_according_to_revision:
+                yield (revid, parents_according_to_index,
+                    parents_according_to_revision)
+
+    def _check_for_inconsistent_revision_parents(self):
+        inconsistencies = list(self._find_inconsistent_revision_parents())
+        if inconsistencies:
+            raise errors.BzrCheckError(
+                "Revision index has inconsistent parents.")
+
+
+class GroupCHKStreamSource(StreamSource):
     """Used when both the source and target repo are GroupCHK repos."""
 
     def __init__(self, from_repository, to_format):
@@ -1240,6 +1240,13 @@ class GroupCHKStreamSource(KnitPackStreamSource):
             self._chk_p_id_roots = None
         yield 'chk_bytes', _get_parent_id_basename_to_file_id_pages()
 
+    def _get_text_stream(self):
+        # Note: We know we don't have to handle adding root keys, because both
+        # the source and target are the identical network name.
+        text_stream = self.from_repository.texts.get_record_stream(
+                        self._text_keys, self._text_fetch_order, False)
+        return ('texts', text_stream)
+
     def get_stream(self, search):
         def wrap_and_count(pb, rc, stream):
             """Yield records from stream while showing progress."""
@@ -1260,17 +1267,20 @@ class GroupCHKStreamSource(KnitPackStreamSource):
             yield (stream_info[0],
                 wrap_and_count(pb, rc, stream_info[1]))
         self._revision_keys = [(rev_id,) for rev_id in revision_ids]
-        self.from_repository.revisions.clear_cache()
-        self.from_repository.signatures.clear_cache()
-        s = self._get_inventory_stream(self._revision_keys)
-        yield (s[0], wrap_and_count(pb, rc, s[1]))
-        self.from_repository.inventories.clear_cache()
         # TODO: The keys to exclude might be part of the search recipe
         # For now, exclude all parents that are at the edge of ancestry, for
         # which we have inventories
         from_repo = self.from_repository
         parent_keys = from_repo._find_parent_keys_of_revisions(
                         self._revision_keys)
+        self.from_repository.revisions.clear_cache()
+        self.from_repository.signatures.clear_cache()
+        # Clear the repo's get_parent_map cache too.
+        self.from_repository._unstacked_provider.disable_cache()
+        self.from_repository._unstacked_provider.enable_cache()
+        s = self._get_inventory_stream(self._revision_keys)
+        yield (s[0], wrap_and_count(pb, rc, s[1]))
+        self.from_repository.inventories.clear_cache()
         for stream_info in self._get_filtered_chk_streams(parent_keys):
             yield (stream_info[0], wrap_and_count(pb, rc, stream_info[1]))
         self.from_repository.chk_bytes.clear_cache()
@@ -1380,7 +1390,8 @@ class RepositoryFormat2a(RepositoryFormatPack):
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return ('Bazaar repository format 2a (needs bzr 1.16 or later)\n')
 
     def get_format_description(self):
@@ -1402,7 +1413,8 @@ class RepositoryFormat2aSubtree(RepositoryFormat2a):
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return ('Bazaar development format 8\n')
 
     def get_format_description(self):
