@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 import errno
 import re
@@ -65,7 +66,8 @@ class ShelfCreator(object):
         """Iterable of tuples describing shelvable changes.
 
         As well as generating the tuples, this updates several members.
-        Tuples may be:
+        Tuples may be::
+
            ('add file', file_id, work_kind, work_path)
            ('delete file', file_id, target_kind, target_path)
            ('rename', file_id, target_path, work_path)
@@ -81,6 +83,9 @@ class ShelfCreator(object):
             # when a tree root was deleted / renamed.
             if kind[0] is None and names[1] == '':
                 continue
+            # Also don't shelve deletion of tree root.
+            if kind[1] is None and names[0] == '':
+                continue
             if kind[0] is None or versioned[0] == False:
                 self.creation[file_id] = (kind[1], names[1], parents[1],
                                           versioned)
