@@ -24,6 +24,7 @@ from bzrlib import (
     osutils,
     tests,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.tests.script import (
     run_script,
     )
@@ -60,8 +61,7 @@ class TestBreakLock(tests.TestCaseWithTransport):
         local_branch = bzrdir.BzrDir.create_branch_convenience('repo/branch')
         local_branch.bind(self.master_branch)
         checkoutdir = bzrdir.BzrDir.create('checkout')
-        branch.BranchReferenceFormat().initialize(
-            checkoutdir, target_branch=local_branch)
+        checkoutdir.set_branch_reference(local_branch)
         self.wt = checkoutdir.create_workingtree()
 
     def test_break_lock_help(self):
@@ -104,15 +104,6 @@ class TestBreakLock(tests.TestCaseWithTransport):
         self.assertRaises(errors.LockBroken, self.master_branch.unlock)
 
 
-class TestBreakLockOldBranch(tests.TestCaseWithTransport):
-
-    def test_break_lock_format_5_bzrdir(self):
-        # break lock on a format 5 bzrdir should just return
-        self.make_branch_and_tree('foo', format=bzrdir.BzrDirFormat5())
-        out, err = self.run_bzr('break-lock foo')
-        self.assertEqual('', out)
-        self.assertEqual('', err)
-
 class TestConfigBreakLock(tests.TestCaseWithTransport):
 
     def setUp(self):
@@ -133,3 +124,20 @@ class TestConfigBreakLock(tests.TestCaseWithTransport):
                      stdin="y\n")
         self.assertRaises(errors.LockBroken, self.config.unlock)
 
+
+class TestSmartServerBreakLock(tests.TestCaseWithTransport):
+
+    def test_simple_branch_break_lock(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        t.branch.lock_write()
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['break-lock', '--force', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertLength(5, self.hpss_calls)
