@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@ from testtools import matchers
 
 from bzrlib import (
     branch,
-    branchbuilder,
     bzrdir,
     conflicts,
     merge_directive,
@@ -35,6 +34,13 @@ from bzrlib import (
     urlutils,
     workingtree,
     )
+from bzrlib.tests import (
+    scenarios,
+    script,
+    )
+
+
+load_tests = scenarios.load_tests_apply_scenarios
 
 
 class TestMerge(tests.TestCaseWithTransport):
@@ -82,45 +88,49 @@ class TestMerge(tests.TestCaseWithTransport):
 
         self.build_tree_contents([('a/hello', 'quuux')])
         # We can't merge when there are in-tree changes
-        os.chdir('a')
-        self.run_bzr('merge ../b', retcode=3)
-        a = workingtree.WorkingTree.open('.')
+        self.run_bzr('merge ../b', retcode=3, working_dir='a')
+        a = workingtree.WorkingTree.open('a')
         a_tip = a.commit("Like an epidemic of u's")
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type blooof',
-                    retcode=3)
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type merge3')
-        a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type weave')
-        a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type lca')
-        a_tree.revert(backups=False)
+
+        def run_merge_then_revert(args, retcode=None, working_dir='a'):
+            self.run_bzr(['merge', '../b', '-r', 'last:1..last:1'] + args,
+                         retcode=retcode, working_dir=working_dir)
+            if retcode != 3:
+                a_tree.revert(backups=False)
+
+        run_merge_then_revert(['--merge-type', 'bloof'], retcode=3)
+        run_merge_then_revert(['--merge-type', 'merge3'])
+        run_merge_then_revert(['--merge-type', 'weave'])
+        run_merge_then_revert(['--merge-type', 'lca'])
         self.run_bzr_error(['Show-base is not supported for this merge type'],
                            'merge ../b -r last:1..last:1 --merge-type weave'
-                           ' --show-base')
+                           ' --show-base', working_dir='a')
         a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1..last:1 --reprocess')
+        self.run_bzr('merge ../b -r last:1..last:1 --reprocess',
+                     working_dir='a')
         a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1')
-        self.check_file_contents('goodbye', 'quux')
+        self.run_bzr('merge ../b -r last:1', working_dir='a')
+        self.check_file_contents('a/goodbye', 'quux')
         # Merging a branch pulls its revision into the tree
-        b = branch.Branch.open('../b')
+        b = branch.Branch.open('b')
         b_tip = b.last_revision()
         self.assertTrue(a.branch.repository.has_revision(b_tip))
         self.assertEqual([a_tip, b_tip], a.get_parent_ids())
         a_tree.revert(backups=False)
-        out, err = self.run_bzr('merge -r revno:1:./hello', retcode=3)
+        out, err = self.run_bzr('merge -r revno:1:./hello', retcode=3,
+                                working_dir='a')
         self.assertTrue("Not a branch" in err)
         self.run_bzr('merge -r revno:%d:./..revno:%d:../b'
-                    %(ancestor,b.revno()))
+                    %(ancestor,b.revno()), working_dir='a')
         self.assertEquals(a.get_parent_ids(),
                           [a.branch.last_revision(), b.last_revision()])
-        self.check_file_contents('goodbye', 'quux')
+        self.check_file_contents('a/goodbye', 'quux')
         a_tree.revert(backups=False)
-        self.run_bzr('merge -r revno:%d:../b'%b.revno())
+        self.run_bzr('merge -r revno:%d:../b'%b.revno(), working_dir='a')
         self.assertEquals(a.get_parent_ids(),
                           [a.branch.last_revision(), b.last_revision()])
         a_tip = a.commit('merged')
-        self.run_bzr('merge ../b -r last:1')
+        self.run_bzr('merge ../b -r last:1', working_dir='a')
         self.assertEqual([a_tip], a.get_parent_ids())
 
     def test_merge_defaults_to_reprocess(self):
@@ -205,14 +215,14 @@ class TestMerge(tests.TestCaseWithTransport):
             ('b/b.txt', 'hello\nsomething\n'),
             ('b/sub/c.txt', 'hello\nsomething\n')])
         b_tree.commit(message='Modified a.txt')
-        os.chdir('b')
-        self.run_bzr('merge ../a/', retcode=1)
-        self.assertPathExists('sub/a.txt.THIS')
-        self.assertPathExists('sub/a.txt.BASE')
-        os.chdir('../a')
-        self.run_bzr('merge ../b/', retcode=1)
-        self.assertPathExists('sub/a.txt.OTHER')
-        self.assertPathExists('sub/a.txt.BASE')
+
+        self.run_bzr('merge ../a/', retcode=1, working_dir='b')
+        self.assertPathExists('b/sub/a.txt.THIS')
+        self.assertPathExists('b/sub/a.txt.BASE')
+
+        self.run_bzr('merge ../b/', retcode=1, working_dir='a')
+        self.assertPathExists('a/sub/a.txt.OTHER')
+        self.assertPathExists('a/sub/a.txt.BASE')
 
     def test_conflict_leaves_base_this_other_files(self):
         tree, other = self.create_conflicting_branches()
@@ -252,37 +262,37 @@ class TestMerge(tests.TestCaseWithTransport):
         branch_b.set_parent(None)
         self.assertEqual(None, branch_b.get_parent())
         # test merge for failure without parent set
-        os.chdir('branch_b')
-        out = self.run_bzr('merge', retcode=3)
+        out = self.run_bzr('merge', retcode=3, working_dir='branch_b')
         self.assertEquals(out,
                 ('','bzr: ERROR: No location specified or remembered\n'))
 
         # test uncommitted changes
-        self.build_tree(['d'])
+        self.build_tree(['branch_b/d'])
         tree_b.add('d')
         self.run_bzr_error(['Working tree ".*" has uncommitted changes'],
-                           'merge')
+                           'merge', working_dir='branch_b')
 
         # merge should now pass and implicitly remember merge location
         tree_b.commit('commit d')
-        out, err = self.run_bzr('merge ../branch_a')
+        out, err = self.run_bzr('merge ../branch_a', working_dir='branch_b')
 
         base = urlutils.local_path_from_url(branch_a.base)
         self.assertEndsWith(err, '+N  b\nAll changes applied successfully.\n')
         self.assertEquals(osutils.abspath(branch_b.get_submit_branch()),
                           osutils.abspath(parent))
         # test implicit --remember when committing new file
-        self.build_tree(['e'])
+        self.build_tree(['branch_b/e'])
         tree_b.add('e')
         tree_b.commit('commit e')
-        out, err = self.run_bzr('merge')
-        self.assertStartsWith(err,
-                          'Merging from remembered submit location %s\n' % (base,))
+        out, err = self.run_bzr('merge', working_dir='branch_b')
+        self.assertStartsWith(
+            err, 'Merging from remembered submit location %s\n' % (base,))
         # re-open tree as external run_bzr modified it
         tree_b = branch_b.bzrdir.open_workingtree()
         tree_b.commit('merge branch_a')
         # test explicit --remember
-        out, err = self.run_bzr('merge ../branch_c --remember')
+        out, err = self.run_bzr('merge ../branch_c --remember',
+                                working_dir='branch_b')
         self.assertEquals(out, '')
         self.assertEquals(err, '+N  c\nAll changes applied successfully.\n')
         self.assertEquals(osutils.abspath(branch_b.get_submit_branch()),
@@ -304,10 +314,8 @@ class TestMerge(tests.TestCaseWithTransport):
 
         self.build_tree_contents([('branch_b/a', 'goodbye')])
         tree_b.commit('message')
-        os.chdir('branch_b')
-        self.run_bzr('bundle ../branch_a -o ../bundle')
-        os.chdir('../branch_a')
-        self.run_bzr('merge ../bundle', retcode=1)
+        self.run_bzr('bundle ../branch_a -o ../bundle', working_dir='branch_b')
+        self.run_bzr('merge ../bundle', retcode=1, working_dir='branch_a')
         testament_a = Testament.from_revision(tree_a.branch.repository,
                                               tree_b.get_parent_ids()[0])
         testament_b = Testament.from_revision(tree_b.branch.repository,
@@ -317,10 +325,10 @@ class TestMerge(tests.TestCaseWithTransport):
         tree_a.set_conflicts(conflicts.ConflictList())
         tree_a.commit('message')
         # it is legal to attempt to merge an already-merged bundle
-        output = self.run_bzr('merge ../bundle')[1]
+        err = self.run_bzr('merge ../bundle', working_dir='branch_a')[1]
         # but it does nothing
         self.assertFalse(tree_a.changes_from(tree_a.basis_tree()).has_changed())
-        self.assertEqual('Nothing to do.\n', output)
+        self.assertEqual('Nothing to do.\n', err)
 
     def test_merge_uncommitted(self):
         """Check that merge --uncommitted behaves properly"""
@@ -333,7 +341,6 @@ class TestMerge(tests.TestCaseWithTransport):
         tree_a.rename_one('file_1', 'file_i')
         tree_a.commit('commit 2')
         tree_a.rename_one('file_2', 'file_ii')
-        ## os.chdir('b')
         self.run_bzr('merge a --uncommitted -d b')
         self.assertPathExists('b/file_1')
         self.assertPathExists('b/file_ii')
@@ -348,10 +355,20 @@ class TestMerge(tests.TestCaseWithTransport):
         tree_a.bzrdir.sprout('tree_b')
         self.build_tree(['tree_a/file1', 'tree_a/file2'])
         tree_a.add(['file1', 'file2'])
-        os.chdir('tree_b')
-        self.run_bzr(['merge', '--uncommitted', '../tree_a/file1'])
-        self.assertPathExists('file1')
-        self.assertPathDoesNotExist('file2')
+        self.run_bzr(['merge', '--uncommitted', '../tree_a/file1'],
+                     working_dir='tree_b')
+        self.assertPathExists('tree_b/file1')
+        self.assertPathDoesNotExist('tree_b/file2')
+
+    def test_merge_nonexistent_file(self):
+        """It should not be possible to merge changes from a file which
+        does not exist."""
+        tree_a = self.make_branch_and_tree('tree_a')
+        self.build_tree_contents([('tree_a/file', 'bar\n')])
+        tree_a.add(['file'])
+        tree_a.commit('commit 1')
+        self.run_bzr_error(('Path\(s\) do not exist: non/existing',),
+                           ['merge', 'non/existing'], working_dir='tree_a')
 
     def pullable_branch(self):
         tree_a = self.make_branch_and_tree('a')
@@ -366,10 +383,9 @@ class TestMerge(tests.TestCaseWithTransport):
 
     def test_merge_pull(self):
         self.pullable_branch()
-        os.chdir('a')
-        (out, err) = self.run_bzr('merge --pull ../b')
+        (out, err) = self.run_bzr('merge --pull ../b', working_dir='a')
         self.assertContainsRe(out, 'Now on revision 2\\.')
-        tree_a = workingtree.WorkingTree.open('.')
+        tree_a = workingtree.WorkingTree.open('a')
         self.assertEqual([self.id2], tree_a.get_parent_ids())
 
     def test_merge_pull_preview(self):
@@ -396,14 +412,13 @@ class TestMerge(tests.TestCaseWithTransport):
         os.unlink('tree_a/file')
         self.build_tree(['tree_a/file/'])
         tree_a.commit('changed file to directory')
-        os.chdir('tree_b')
-        self.run_bzr('merge ../tree_a')
-        self.assertEqual('directory', osutils.file_kind('file'))
+        self.run_bzr('merge ../tree_a', working_dir='tree_b')
+        self.assertEqual('directory', osutils.file_kind('tree_b/file'))
         tree_b.revert()
-        self.assertEqual('file', osutils.file_kind('file'))
-        self.build_tree_contents([('file', 'content_2')])
+        self.assertEqual('file', osutils.file_kind('tree_b/file'))
+        self.build_tree_contents([('tree_b/file', 'content_2')])
         tree_b.commit('content change')
-        self.run_bzr('merge ../tree_a', retcode=1)
+        self.run_bzr('merge ../tree_a', retcode=1, working_dir='tree_b')
         self.assertEqual(tree_b.conflicts(),
                          [conflicts.ContentsConflict('file',
                                                      file_id='file-id')])
@@ -534,6 +549,7 @@ class TestMerge(tests.TestCaseWithTransport):
 
     def test_merge_from_submit(self):
         tree_a = self.make_branch_and_tree('a')
+        tree_a.commit('test')
         tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
         tree_c = tree_a.bzrdir.sprout('c').open_workingtree()
         out, err = self.run_bzr(['merge', '-d', 'c'])
@@ -544,6 +560,7 @@ class TestMerge(tests.TestCaseWithTransport):
 
     def test_remember_sets_submit(self):
         tree_a = self.make_branch_and_tree('a')
+        tree_a.commit('rev1')
         tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
         self.assertIs(tree_b.branch.get_submit_branch(), None)
 
@@ -556,6 +573,23 @@ class TestMerge(tests.TestCaseWithTransport):
         self.assertEqual(tree_b.branch.get_submit_branch(),
                          tree_a.bzrdir.root_transport.base)
 
+    def test_no_remember_dont_set_submit(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/file', "a\n")])
+        tree_a.add('file')
+        tree_a.commit('rev1')
+        tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
+        self.assertIs(tree_b.branch.get_submit_branch(), None)
+
+        # Remember should not happen if using default from parent
+        out, err = self.run_bzr(['merge', '-d', 'b', '--no-remember'])
+        self.assertEquals(None, tree_b.branch.get_submit_branch())
+
+        # Remember should not happen if user supplies location but ask for not
+        # remembering it
+        out, err = self.run_bzr(['merge', '-d', 'b', '--no-remember', 'a'])
+        self.assertEqual(None, tree_b.branch.get_submit_branch())
+
     def test_weave_cherrypick(self):
         this_tree = self.make_branch_and_tree('this')
         self.build_tree_contents([('this/file', "a\n")])
@@ -625,24 +659,67 @@ class TestMerge(tests.TestCaseWithTransport):
 
     def test_merge_interactive_unlocks_branch(self):
         this = self.make_branch_and_tree('this')
-        other = self.make_branch_and_tree('other')
-        other.commit('empty commit')
+        this.commit('empty commit')
+        other = this.bzrdir.sprout('other').open_workingtree()
+        other.commit('empty commit 2')
         self.run_bzr('merge -i -d this other')
         this.lock_write()
         this.unlock()
 
-    def test_merge_reversed_revision_range(self):
-        tree = self.make_branch_and_tree(".")
+    def test_merge_fetches_tags(self):
+        """Tags are updated by merge, and revisions named in those tags are
+        fetched.
+        """
+        # Make a source, sprout a target off it
+        builder = self.make_branch_builder('source')
+        builder.build_commit(message="Rev 1", rev_id='rev-1')
+        source = builder.get_branch()
+        target_bzrdir = source.bzrdir.sprout('target')
+        # Add a non-ancestry tag to source
+        builder.build_commit(message="Rev 2a", rev_id='rev-2a')
+        source.tags.set_tag('tag-a', 'rev-2a')
+        source.set_last_revision_info(1, 'rev-1')
+        source.get_config_stack().set('branch.fetch_tags', True)
+        builder.build_commit(message="Rev 2b", rev_id='rev-2b')
+        # Merge from source
+        self.run_bzr('merge -d target source')
+        target = target_bzrdir.open_branch()
+        # The tag is present, and so is its revision.
+        self.assertEqual('rev-2a', target.tags.lookup_tag('tag-a'))
+        target.repository.get_revision('rev-2a')
+
+
+class TestMergeRevisionRange(tests.TestCaseWithTransport):
+
+    scenarios = (('whole-tree', dict(context='.')),
+                 ('file-only', dict(context='a')))
+
+    def setUp(self):
+        super(TestMergeRevisionRange, self).setUp()
+        self.tree = self.make_branch_and_tree(".")
+        self.tree.commit('initial commit')
         for f in ("a", "b"):
             self.build_tree([f])
-            tree.add(f)
-            tree.commit("added "+f)
-        for context in (".", "", "a"):
-            self.run_bzr("merge -r 1..0 " + context)
-            self.assertPathDoesNotExist("a")
-            tree.revert()
-            self.assertPathExists("a")
+            self.tree.add(f)
+            self.tree.commit("added " + f)
 
+    def test_merge_reversed_revision_range(self):
+        self.run_bzr("merge -r 2..1 " + self.context)
+        self.assertPathDoesNotExist("a")
+        self.assertPathExists("b")
+
+
+class TestMergeScript(script.TestCaseWithTransportAndScript):
+    def test_merge_empty_branch(self):
+        source = self.make_branch_and_tree('source')
+        self.build_tree(['source/a'])
+        source.add('a')
+        source.commit('Added a', rev_id='rev1')
+        target = self.make_branch_and_tree('target')
+        self.run_script("""\
+$ bzr merge -d target source
+2>bzr: ERROR: Merging into empty branches not currently supported, https://bugs.launchpad.net/bzr/+bug/308562
+""")
 
 class TestMergeForce(tests.TestCaseWithTransport):
 
