@@ -17,27 +17,15 @@
 """Black box tests for the reconcile command."""
 
 
-import bzrlib
-import bzrlib.bzrdir as bzrdir
-from bzrlib.inventory import Inventory
-import bzrlib.repository as repository
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.tests import TestUIFactory
-from bzrlib.transport import get_transport
-import bzrlib.ui as ui
+from bzrlib import (
+    bzrdir,
+    inventory,
+    tests,
+    )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
-class TrivialTest(TestCaseWithTransport):
-
-    def setUp(self):
-        super(TrivialTest, self).setUp()
-        self.old_format = bzrdir.BzrDirFormat.get_default_format()
-        self.old_ui_factory = ui.ui_factory
-        self.addCleanup(self.restoreDefaults)
-        ui.ui_factory = TestUIFactory()
-
-    def restoreDefaults(self):
-        ui.ui_factory = self.old_ui_factory
+class TrivialTest(tests.TestCaseWithTransport):
 
     def test_trivial_reconcile(self):
         t = bzrdir.BzrDir.create_standalone_workingtree('.')
@@ -60,7 +48,7 @@ class TrivialTest(TestCaseWithTransport):
         t = bzrdir.BzrDir.create_standalone_workingtree('.')
         # an empty inventory with no revision will trigger reconciliation.
         repo = t.branch.repository
-        inv = Inventory(revision_id='missing')
+        inv = inventory.Inventory(revision_id='missing')
         inv.root.revision='missing'
         repo.lock_write()
         repo.start_write_group()
@@ -84,3 +72,20 @@ class TrivialTest(TestCaseWithTransport):
                      does_backup_text))
         self.assertEqualDiff(expected, out)
         self.assertEqualDiff(err, "")
+
+
+class TestSmartServerReconcile(tests.TestCaseWithTransport):
+
+    def test_simple_reconcile(self):
+        self.setup_smart_server_with_call_log()
+        self.make_branch('branch')
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['reconcile', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(10, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
