@@ -16,7 +16,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import sys
 from cStringIO import StringIO
 
 from bzrlib import (
@@ -25,12 +24,16 @@ from bzrlib import (
     merge_directive,
     tests,
     )
+from bzrlib.controldir import ControlDir
 from bzrlib.bundle import serializer
 from bzrlib.transport import memory
-from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests import (
+    scenarios,
+    )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
-load_tests = load_tests_apply_scenarios
+load_tests = scenarios.load_tests_apply_scenarios
 
 
 class TestSendMixin(object):
@@ -186,7 +189,7 @@ class TestSend(tests.TestCaseWithTransport, TestSendMixin):
 
     def test_note_revisions(self):
         stderr = self.run_send([])[1]
-        self.assertEndsWith(stderr, '\nBundling 1 revision(s).\n')
+        self.assertEndsWith(stderr, '\nBundling 1 revision.\n')
 
     def test_mailto_option(self):
         b = branch.Branch.open('branch')
@@ -198,7 +201,7 @@ class TestSend(tests.TestCaseWithTransport, TestSendMixin):
         self.run_send([])
         self.run_bzr_error(('Unknown mail client: bogus',),
                            'send -f branch --mail-to jrandom@example.org')
-        b.get_config().set_user_option('submit_to', 'jrandom@example.org')
+        b.get_config_stack().set('submit_to', 'jrandom@example.org')
         self.run_bzr_error(('Unknown mail client: bogus',),
                            'send -f branch')
 
@@ -207,10 +210,8 @@ class TestSend(tests.TestCaseWithTransport, TestSendMixin):
         b = branch.Branch.open('branch')
         b.get_config().set_user_option('mail_client', 'bogus')
         parent = branch.Branch.open('parent')
-        parent.get_config().set_user_option('child_submit_to',
-                           'somebody@example.org')
-        self.run_bzr_error(('Unknown mail client: bogus',),
-                           'send -f branch')
+        parent.get_config_stack().set('child_submit_to', 'somebody@example.org')
+        self.run_bzr_error(('Unknown mail client: bogus',), 'send -f branch')
 
     def test_format(self):
         md = self.get_MD(['--format=4'])
@@ -228,12 +229,12 @@ class TestSend(tests.TestCaseWithTransport, TestSendMixin):
                             'send -f branch -o- --format=0.999')[0]
 
     def test_format_child_option(self):
-        parent_config = branch.Branch.open('parent').get_config()
-        parent_config.set_user_option('child_submit_format', '4')
+        parent_config = branch.Branch.open('parent').get_config_stack()
+        parent_config.set('child_submit_format', '4')
         md = self.get_MD([])
         self.assertIs(merge_directive.MergeDirective2, md.__class__)
 
-        parent_config.set_user_option('child_submit_format', '0.9')
+        parent_config.set('child_submit_format', '0.9')
         md = self.get_MD([])
         self.assertFormatIs('# Bazaar revision bundle v0.9', md)
 
@@ -241,7 +242,7 @@ class TestSend(tests.TestCaseWithTransport, TestSendMixin):
         self.assertFormatIs('# Bazaar revision bundle v0.9', md)
         self.assertIs(merge_directive.MergeDirective, md.__class__)
 
-        parent_config.set_user_option('child_submit_format', '0.999')
+        parent_config.set('child_submit_format', '0.999')
         self.run_bzr_error(["No such send format '0.999'"],
                             'send -f branch -o-')[0]
 
@@ -293,8 +294,8 @@ class TestSendStrictMixin(TestSendMixin):
     def set_config_send_strict(self, value):
         # set config var (any of bazaar.conf, locations.conf, branch.conf
         # should do)
-        conf = self.local_tree.branch.get_config()
-        conf.set_user_option('send_strict', value)
+        conf = self.local_tree.branch.get_config_stack()
+        conf.set('send_strict', value)
 
     def assertSendFails(self, args):
         out, err = self.run_send(args, rc=3, err_re=self._default_errors)
@@ -308,7 +309,10 @@ class TestSendStrictMixin(TestSendMixin):
         if revs is None:
             revs = self._default_sent_revs
         out, err = self.run_send(args, err_re=err_re)
-        bundling_revs = 'Bundling %d revision(s).\n' % len(revs)
+        if len(revs) == 1:
+            bundling_revs = 'Bundling %d revision.\n'% len(revs)
+        else:
+            bundling_revs = 'Bundling %d revisions.\n' % len(revs)
         if with_warning:
             self.assertContainsRe(err, self._default_additional_warning)
             self.assertEndsWith(err, bundling_revs)
@@ -327,6 +331,10 @@ class TestSendStrictWithoutChanges(tests.TestCaseWithTransport,
         super(TestSendStrictWithoutChanges, self).setUp()
         self.make_parent_and_local_branches()
 
+    def test_send_without_workingtree(self):
+        ControlDir.open("local").destroy_workingtree()
+        self.assertSendSucceeds([])
+
     def test_send_default(self):
         self.assertSendSucceeds([])
 
@@ -346,7 +354,7 @@ class TestSendStrictWithoutChanges(tests.TestCaseWithTransport,
 
 
 class TestSendStrictWithChanges(tests.TestCaseWithTransport,
-                                   TestSendStrictMixin):
+                                TestSendStrictMixin):
 
     # These are textually the same as test_push.strict_push_change_scenarios,
     # but since the functions are reimplemented here, the definitions are left
@@ -433,3 +441,27 @@ class TestSendStrictWithChanges(tests.TestCaseWithTransport,
 class TestBundleStrictWithoutChanges(TestSendStrictWithoutChanges):
 
     _default_command = ['bundle-revisions', '../parent']
+
+
+class TestSmartServerSend(tests.TestCaseWithTransport):
+
+    def test_send(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        local = t.bzrdir.sprout('local-branch').open_workingtree()
+        self.build_tree_contents([('branch/foo', 'thenewcontents')])
+        local.commit("anothermessage")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(
+            ['send', '-o', 'x.diff', self.get_url('branch')], working_dir='local-branch')
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(9, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
