@@ -21,24 +21,56 @@ import stat
 from bzrlib import (
     bzrdir,
     controldir,
-    transport,
+    lockable_files,
+    ui,
+    urlutils,
     )
 from bzrlib.tests import (
     features,
     TestCaseWithTransport,
     )
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.repofmt.knitrepo import (
-    RepositoryFormatKnit1,
-    )
+from bzrlib.repofmt.knitpack_repo import RepositoryFormatKnitPack1
+
+
+class OldBzrDir(bzrdir.BzrDirMeta1):
+    """An test bzr dir implementation"""
+
+    def needs_format_conversion(self, format):
+        return not isinstance(format, self.__class__)
+
+
+class ConvertOldTestToMeta(controldir.Converter):
+    """A trivial converter, used for testing."""
+
+    def convert(self, to_convert, pb):
+        ui.ui_factory.note('starting upgrade from old test format to 2a')
+        to_convert.control_transport.put_bytes(
+            'branch-format',
+            bzrdir.BzrDirMetaFormat1().get_format_string(),
+            mode=to_convert._get_file_mode())
+        return bzrdir.BzrDir.open(to_convert.user_url)
+
+
+class OldBzrDirFormat(bzrdir.BzrDirMetaFormat1):
+
+    _lock_class = lockable_files.TransportLock
+
+    def get_converter(self, format=None):
+        return ConvertOldTestToMeta()
+
+    @classmethod
+    def get_format_string(cls):
+        return "Ancient Test Format"
+
+    def _open(self, transport):
+        return OldBzrDir(transport, self)
 
 
 class TestWithUpgradableBranches(TestCaseWithTransport):
 
     def setUp(self):
         super(TestWithUpgradableBranches, self).setUp()
-        self.addCleanup(controldir.ControlDirFormat._set_default_format,
-                        controldir.ControlDirFormat.get_default_format())
 
     def make_current_format_branch_and_checkout(self):
         current_tree = self.make_branch_and_tree('current_format_branch',
@@ -46,19 +78,10 @@ class TestWithUpgradableBranches(TestCaseWithTransport):
         current_tree.branch.create_checkout(
             self.get_url('current_format_checkout'), lightweight=True)
 
-    def make_format_5_branch(self):
-        # setup a format 5 branch we can upgrade from.
-        path = 'format_5_branch'
-        self.make_branch_and_tree(path, format=bzrdir.BzrDirFormat5())
-        return path
-
-    def make_metadir_weave_branch(self):
-        self.make_branch_and_tree('metadir_weave_branch', format='metaweave')
-
     def test_readonly_url_error(self):
-        path = self.make_format_5_branch()
+        self.make_branch_and_tree("old_format_branch", format="knit")
         (out, err) = self.run_bzr(
-            ['upgrade', self.get_readonly_url(path)], retcode=3)
+            ['upgrade', self.get_readonly_url("old_format_branch")], retcode=3)
         err_msg = 'Upgrade URL cannot work with readonly URLs.'
         self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
                              % (err_msg, err_msg),
@@ -67,31 +90,28 @@ class TestWithUpgradableBranches(TestCaseWithTransport):
     def test_upgrade_up_to_date(self):
         self.make_current_format_branch_and_checkout()
         # when up to date we should get a message to that effect
-        (out, err) = self.run_bzr('upgrade current_format_branch', retcode=3)
-        err_msg = ('The branch format %s is already at the most recent format.'
-                   % ('Meta directory format 1'))
-        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
-                             % (err_msg, err_msg),
-                             err)
+        burl = self.get_transport('current_format_branch').local_abspath(".")
+        (out, err) = self.run_bzr('upgrade current_format_branch', retcode=0)
+        self.assertEqual(
+            'Upgrading branch %s/ ...\n'
+            'The branch format %s is already at the most recent format.\n'
+            % (burl, 'Meta directory format 1'),
+            out)
 
     def test_upgrade_up_to_date_checkout_warns_branch_left_alone(self):
         self.make_current_format_branch_and_checkout()
         # when upgrading a checkout, the branch location and a suggestion
         # to upgrade it should be emitted even if the checkout is up to
         # date
-        burl = self.get_transport('current_format_branch').base
-        curl = self.get_transport('current_format_checkout').base
-        (out, err) = self.run_bzr('upgrade current_format_checkout', retcode=3)
+        burl = self.get_transport('current_format_branch').local_abspath(".")
+        curl = self.get_transport('current_format_checkout').local_abspath(".")
+        (out, err) = self.run_bzr('upgrade current_format_checkout', retcode=0)
         self.assertEqual(
-            'Upgrading branch %s ...\nThis is a checkout.'
-            ' The branch (%s) needs to be upgraded separately.\n'
-            % (curl, burl),
+            'Upgrading branch %s/ ...\nThis is a checkout.'
+            ' The branch (%s/) needs to be upgraded separately.\n'
+            'The branch format %s is already at the most recent format.\n'
+            % (curl, burl, 'Meta directory format 1'),
             out)
-        msg = 'The branch format %s is already at the most recent format.' % (
-            'Meta directory format 1')
-        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
-                             % (msg, msg),
-                             err)
 
     def test_upgrade_checkout(self):
         # upgrading a checkout should work
@@ -106,59 +126,68 @@ class TestWithUpgradableBranches(TestCaseWithTransport):
         # upgrading a branch in a repo should warn about not upgrading the repo
         pass
 
-    def test_upgrade_explicit_metaformat(self):
-        # users can force an upgrade to metadir format.
-        path = self.make_format_5_branch()
-        url = self.get_transport(path).base
+    def test_upgrade_control_dir(self):
+        old_format = OldBzrDirFormat()
+        self.addCleanup(bzrdir.BzrProber.formats.remove,
+            old_format.get_format_string())
+        bzrdir.BzrProber.formats.register(old_format.get_format_string(),
+            old_format)
+        self.addCleanup(controldir.ControlDirFormat._set_default_format,
+                        controldir.ControlDirFormat.get_default_format())
+
+        # setup an old format branch we can upgrade from.
+        path = 'old_format_branch'
+        self.make_branch_and_tree(path, format=old_format)
+        transport = self.get_transport(path)
+        url = transport.base
+        display_url = transport.local_abspath('.')
         # check --format takes effect
-        controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
+        controldir.ControlDirFormat._set_default_format(old_format)
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
-            ['upgrade', '--format=metaweave', url])
-        self.assertEqualDiff("""Upgrading branch %s ...
-starting upgrade of %s
-making backup of %s.bzr
-  to %s%s
-starting upgrade from format 5 to 6
-adding prefixes to weaves
-adding prefixes to revision-store
-starting upgrade from format 6 to metadir
+            ['upgrade', '--format=2a', url])
+        self.assertEqualDiff("""Upgrading branch %s/ ...
+starting upgrade of %s/
+making backup of %s/.bzr
+  to %s/%s
+starting upgrade from old test format to 2a
 finished
-""" % (url, url, url, url, backup_dir), out)
+""" % (display_url, display_url, display_url, display_url, backup_dir), out)
         self.assertEqualDiff("", err)
         self.assertTrue(isinstance(
             bzrdir.BzrDir.open(self.get_url(path))._format,
             bzrdir.BzrDirMetaFormat1))
 
     def test_upgrade_explicit_knit(self):
-        # users can force an upgrade to knit format from a metadir weave
-        # branch
-        self.make_metadir_weave_branch()
-        url = self.get_transport('metadir_weave_branch').base
+        # users can force an upgrade to knit format from a metadir pack 0.92
+        # branch to a 2a branch.
+        self.make_branch_and_tree('branch', format='knit')
+        transport = self.get_transport('branch')
+        url = transport.base
+        display_url = transport.local_abspath('.')
         # check --format takes effect
-        controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
-            ['upgrade', '--format=knit', url])
-        self.assertEqualDiff("""Upgrading branch %s ...
-starting upgrade of %s
-making backup of %s.bzr
-  to %s%s
+            ['upgrade', '--format=pack-0.92', url])
+        self.assertEqualDiff("""Upgrading branch %s/ ...
+starting upgrade of %s/
+making backup of %s/.bzr
+  to %s/%s
 starting repository conversion
 repository converted
 finished
-""" % (url, url, url, url, backup_dir),
+""" % (display_url, display_url, display_url, display_url, backup_dir),
                              out)
         self.assertEqualDiff("", err)
-        converted_dir = bzrdir.BzrDir.open(self.get_url('metadir_weave_branch'))
+        converted_dir = bzrdir.BzrDir.open(self.get_url('branch'))
         self.assertTrue(isinstance(converted_dir._format,
                                    bzrdir.BzrDirMetaFormat1))
         self.assertTrue(isinstance(converted_dir.open_repository()._format,
-                                   RepositoryFormatKnit1))
+                                   RepositoryFormatKnitPack1))
 
     def test_upgrade_repo(self):
-        self.run_bzr('init-repository --format=metaweave repo')
-        self.run_bzr('upgrade --format=knit repo')
+        self.run_bzr('init-repository --format=pack-0.92 repo')
+        self.run_bzr('upgrade --format=2a repo')
 
     def assertLegalOption(self, option_str):
         # Confirm that an option is legal. (Lower level tests are
@@ -195,51 +224,51 @@ finished
         self.assertTrue(new_perms == old_perms)
 
     def test_upgrade_with_existing_backup_dir(self):
-        path = self.make_format_5_branch()
-        t = self.get_transport(path)
+        self.make_branch_and_tree("old_format_branch", format="knit")
+        t = self.get_transport("old_format_branch")
         url = t.base
-        controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
+        display_url = t.local_abspath('.')
         backup_dir1 = 'backup.bzr.~1~'
         backup_dir2 = 'backup.bzr.~2~'
         # explicitly create backup_dir1. bzr should create the .~2~ directory
         # as backup
         t.mkdir(backup_dir1)
         (out, err) = self.run_bzr(
-            ['upgrade', '--format=metaweave', url])
-        self.assertEqualDiff("""Upgrading branch %s ...
-starting upgrade of %s
-making backup of %s.bzr
-  to %s%s
-starting upgrade from format 5 to 6
-adding prefixes to weaves
-adding prefixes to revision-store
-starting upgrade from format 6 to metadir
+            ['upgrade', '--format=2a', url])
+        self.assertEqualDiff("""Upgrading branch %s/ ...
+starting upgrade of %s/
+making backup of %s/.bzr
+  to %s/%s
+starting repository conversion
+repository converted
 finished
-""" % (url, url, url, url, backup_dir2), out)
+""" % (display_url, display_url, display_url, display_url, backup_dir2), out)
         self.assertEqualDiff("", err)
         self.assertTrue(isinstance(
-            bzrdir.BzrDir.open(self.get_url(path))._format,
+            bzrdir.BzrDir.open(self.get_url("old_format_branch"))._format,
             bzrdir.BzrDirMetaFormat1))
         self.assertTrue(t.has(backup_dir2))
 
+
 class SFTPTests(TestCaseWithSFTPServer):
     """Tests for upgrade over sftp."""
 
     def test_upgrade_url(self):
-        self.run_bzr('init --format=weave')
+        self.run_bzr('init --format=pack-0.92')
         t = self.get_transport()
         url = t.base
-        out, err = self.run_bzr(['upgrade', '--format=knit', url])
+        display_url = urlutils.unescape_for_display(url,
+            'utf-8')
+        out, err = self.run_bzr(['upgrade', '--format=2a', url])
         backup_dir = 'backup.bzr.~1~'
         self.assertEqualDiff("""Upgrading branch %s ...
 starting upgrade of %s
 making backup of %s.bzr
   to %s%s
-starting upgrade from format 6 to metadir
 starting repository conversion
 repository converted
 finished
-""" % (url, url, url, url,backup_dir), out)
+""" % (display_url, display_url, display_url, display_url, backup_dir), out)
         self.assertEqual('', err)
 
 
