@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2008 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,17 +29,12 @@ So if the directory is group writable, the files and subdirs should be as well.
 import os
 import sys
 import stat
-from StringIO import StringIO
 
 from bzrlib import tests
-from bzrlib.branch import Branch
+from bzrlib.branch import BzrBranch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.lockable_files import LockableFiles
 from bzrlib.remote import RemoteBranchFormat
-from bzrlib.tests.test_permissions import chmod_r, check_mode_r
-from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.transport import get_transport
-from bzrlib.workingtree import WorkingTree
+from bzrlib.tests.test_permissions import check_mode_r
 
 
 class _NullPermsStat(object):
@@ -64,10 +59,13 @@ class TestPermissions(tests.TestCaseWithTransport):
                                           ' permission logic')
         if sys.platform == 'win32':
             raise tests.TestNotApplicable('chmod has no effect on win32')
-        # also, these are BzrBranch format specific things..
         os.mkdir('a')
         mode = stat.S_IMODE(os.stat('a').st_mode)
         t = self.make_branch_and_tree('.')
+        # also, these are BzrBranch format specific things..
+        if not isinstance(t.branch, BzrBranch):
+            raise tests.TestNotApplicable(
+                "Only applicable to bzr branches")
         b = t.branch
         self.assertEqualMode(mode, b.bzrdir._get_dir_mode())
         self.assertEqualMode(mode & ~07111, b.bzrdir._get_file_mode())
@@ -91,14 +89,17 @@ class TestPermissions(tests.TestCaseWithTransport):
                                           ' permission logic')
         if sys.platform == 'win32':
             raise tests.TestNotApplicable('chmod has no effect on win32')
-        elif sys.platform == 'darwin' or sys.platform.startswith('freebsd'):
-            # OS X (and FreeBSD) create temp dirs with the 'wheel' group, which
-            # users are not likely to be in, and this prevents us from setting
-            # the sgid bit
+        elif sys.platform == 'darwin' or 'freebsd' in sys.platform:
+            # FreeBSD-based platforms create temp dirs with the 'wheel' group,
+            # which users are not likely to be in, and this prevents us
+            # from setting the sgid bit
             os.chown(self.test_dir, os.getuid(), os.getgid())
-        # also, these are BzrBranch format specific things..
         t = self.make_branch_and_tree('.')
         b = t.branch
+        # also, these are BzrBranch format specific things..
+        if not isinstance(b, BzrBranch):
+            raise tests.TestNotApplicable(
+                "Only applicable to bzr branches")
         os.mkdir('b')
         os.chmod('b', 02777)
         b = self.make_branch('b')
