@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ from bzrlib import (
 from bzrlib.errors import (
     NoSuchRevision,
     )
-from bzrlib.graph import (
+from bzrlib.vf_search import (
     SearchResult,
     )
 from bzrlib.revision import (
@@ -76,7 +76,7 @@ class TestInterRepository(TestCaseWithInterRepository):
             tree.lock_read()
             self.addCleanup(tree.unlock)
             tree.get_file_text('file1')
-            for file_id in tree:
+            for file_id in tree.all_file_ids():
                 if tree.inventory[file_id].kind == "file":
                     tree.get_file(file_id).read()
 
@@ -451,18 +451,13 @@ class TestInterRepository(TestCaseWithInterRepository):
     def test_fetch_missing_revision_same_location_fails(self):
         repo_a = self.make_repository('.')
         repo_b = repository.Repository.open('.')
-        try:
-            self.assertRaises(errors.NoSuchRevision, repo_b.fetch, repo_a, revision_id='XXX')
-        except errors.LockError, e:
-            check_old_format_lock_error(self.repository_format)
+        self.assertRaises(errors.NoSuchRevision,
+            repo_b.fetch, repo_a, revision_id='XXX')
 
     def test_fetch_same_location_trivial_works(self):
         repo_a = self.make_repository('.')
         repo_b = repository.Repository.open('.')
-        try:
-            repo_a.fetch(repo_b)
-        except errors.LockError, e:
-            check_old_format_lock_error(self.repository_format)
+        repo_a.fetch(repo_b)
 
     def test_fetch_missing_text_other_location_fails(self):
         source_tree = self.make_branch_and_tree('source')
@@ -515,7 +510,7 @@ class TestInterRepository(TestCaseWithInterRepository):
         from_tree.commit('foo', rev_id='foo-id')
         to_repo = self.make_to_repository('to')
         to_repo.fetch(from_tree.branch.repository)
-        recorded_inv_sha1 = to_repo.get_inventory_sha1('foo-id')
+        recorded_inv_sha1 = to_repo.get_revision('foo-id').inventory_sha1
         to_repo.lock_read()
         self.addCleanup(to_repo.unlock)
         stream = to_repo.inventories.get_record_stream([('foo-id',)],
