@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,19 +14,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from StringIO import StringIO
-
 import bzrlib
 from bzrlib import (
     errors,
     lockdir,
     osutils,
+    transport,
     )
-from bzrlib.errors import BzrBadParameterNotString, NoSuchFile, ReadOnlyError
 from bzrlib.lockable_files import LockableFiles, TransportLock
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import (
     TestCaseInTempDir,
     TestNotApplicable,
@@ -37,7 +32,6 @@ from bzrlib.transactions import (PassThroughTransaction,
                                  ReadOnlyTransaction,
                                  WriteTransaction,
                                  )
-from bzrlib.transport import get_transport
 
 
 # these tests are applied in each parameterized suite for LockableFiles
@@ -279,9 +273,9 @@ class TestLockableFiles_TransportLock(TestCaseInTempDir,
 
     def setUp(self):
         TestCaseInTempDir.setUp(self)
-        transport = get_transport('.')
-        transport.mkdir('.bzr')
-        self.sub_transport = transport.clone('.bzr')
+        t = transport.get_transport_from_path('.')
+        t.mkdir('.bzr')
+        self.sub_transport = t.clone('.bzr')
         self.lockable = self.get_lockable()
         self.lockable.create_lock()
 
@@ -304,7 +298,7 @@ class TestLockableFiles_LockDir(TestCaseInTempDir,
 
     def setUp(self):
         TestCaseInTempDir.setUp(self)
-        self.transport = get_transport('.')
+        self.transport = transport.get_transport_from_path('.')
         self.lockable = self.get_lockable()
         # the lock creation here sets mode - test_permissions on branch
         # tests that implicitly, but it might be a good idea to factor
@@ -347,7 +341,7 @@ class TestLockableFiles_RemoteLockDir(TestCaseWithSmartMedium,
         # in test_remote and test_smart as usual.
         b = self.make_branch('foo')
         self.addCleanup(b.bzrdir.transport.disconnect)
-        self.transport = get_transport('.')
+        self.transport = transport.get_transport_from_path('.')
         self.lockable = self.get_lockable()
 
     def get_lockable(self):
