@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,14 +12,16 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from bzrlib import (
     branch as _mod_branch,
+    bzrdir,
     errors,
     reconfigure,
     repository,
     tests,
+    vf_repository,
     workingtree,
     )
 
@@ -43,6 +45,18 @@ class TestReconfigure(tests.TestCaseWithTransport):
         self.assertRaises(errors.NoWorkingTree, workingtree.WorkingTree.open,
                           'tree')
 
+    def test_tree_with_pending_merge_to_branch(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.commit('unchanged')
+        other_tree = tree.bzrdir.sprout('other').open_workingtree()
+        other_tree.commit('mergeable commit')
+        tree.merge_from_branch(other_tree.branch)
+        reconfiguration = reconfigure.Reconfigure.to_branch(tree.bzrdir)
+        self.assertRaises(errors.UncommittedChanges, reconfiguration.apply)
+        reconfiguration.apply(force=True)
+        self.assertRaises(errors.NoWorkingTree, workingtree.WorkingTree.open,
+                          'tree')
+
     def test_branch_to_branch(self):
         branch = self.make_branch('branch')
         self.assertRaises(errors.AlreadyBranch,
@@ -245,12 +259,12 @@ class TestReconfigure(tests.TestCaseWithTransport):
     def test_branch_to_lightweight_checkout_failure(self):
         parent, child, reconfiguration = \
             self.prepare_branch_to_lightweight_checkout()
-        old_Repository_fetch = repository.Repository.fetch
-        repository.Repository.fetch = None
+        old_Repository_fetch = vf_repository.VersionedFileRepository.fetch
+        vf_repository.VersionedFileRepository.fetch = None
         try:
             self.assertRaises(TypeError, reconfiguration.apply)
         finally:
-            repository.Repository.fetch = old_Repository_fetch
+            vf_repository.VersionedFileRepository.fetch = old_Repository_fetch
         child = _mod_branch.Branch.open('child')
         self.assertContainsRe(child.base, 'child/$')
 
@@ -417,3 +431,15 @@ class TestReconfigure(tests.TestCaseWithTransport):
             reconfigure.Reconfigure.set_repository_trees, tree.bzrdir, None)
         self.assertContainsRe(str(e),
             r"Requested reconfiguration of '.*' is not supported.")
+
+    def test_lightweight_checkout_to_tree_preserves_reference_locations(self):
+        format = bzrdir.format_registry.make_bzrdir('1.9')
+        format.set_branch_format(_mod_branch.BzrBranchFormat8())
+        tree = self.make_branch_and_tree('tree', format=format)
+        tree.branch.set_reference_info('file_id', 'path', '../location')
+        checkout = tree.branch.create_checkout('checkout', lightweight=True)
+        reconfiguration = reconfigure.Reconfigure.to_tree(checkout.bzrdir)
+        reconfiguration.apply()
+        checkout_branch = checkout.bzrdir.open_branch()
+        self.assertEqual(('path', '../location'),
+                         checkout_branch.get_reference_info('file_id'))
