@@ -26,31 +26,8 @@ from bzrlib import (
     )
 from bzrlib.tests import TestCase
 from bzrlib.inventory import Inventory
-from bzrlib.xml4 import serializer_v4
 import bzrlib.xml5
 
-_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
-<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
-<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
-<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
-</inventory>"""
-
-
-_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
-    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
-    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
-    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
-    timestamp="1125907235.212"
-    timezone="36000">
-<message>- start splitting code for xml (de)serialization away from objects
-  preparatory to supporting multiple formats by a single library
-</message>
-<parents>
-<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
-</parents>
-</revision>
-"""
-
 _revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
     inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
     revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
@@ -217,26 +194,6 @@ _inventory_v5b = """<inventory format="5" revision_id="a-rev-id">
 class TestSerializer(TestCase):
     """Test XML serialization"""
 
-    def test_canned_inventory(self):
-        """Test unpacked a canned inventory v4 file."""
-        inp = StringIO(_working_inventory_v4)
-        inv = serializer_v4.read_inventory(inp)
-        self.assertEqual(len(inv), 4)
-        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
-
-    def test_unpack_revision(self):
-        """Test unpacking a canned revision v4"""
-        inp = StringIO(_revision_v4)
-        rev = serializer_v4.read_revision(inp)
-        eq = self.assertEqual
-        eq(rev.committer,
-           "Martin Pool <mbp@sourcefrog.net>")
-        eq(rev.inventory_id,
-           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
-        eq(len(rev.parent_ids), 1)
-        eq(rev.parent_ids[0],
-           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
-
     def test_unpack_revision_5(self):
         """Test unpacking a canned revision v5"""
         inp = StringIO(_revision_v5)
@@ -551,31 +508,37 @@ class TestEncodeAndEscape(TestCase):
     def setUp(self):
         TestCase.setUp(self)
         # Keep the cache clear before and after the test
-        bzrlib.xml8._ensure_utf8_re()
-        bzrlib.xml8._clear_cache()
-        self.addCleanup(bzrlib.xml8._clear_cache)
+        bzrlib.xml_serializer._clear_cache()
+        self.addCleanup(bzrlib.xml_serializer._clear_cache)
 
     def test_simple_ascii(self):
         # _encode_and_escape always appends a final ", because these parameters
         # are being used in xml attributes, and by returning it now, we have to
         # do fewer string operations later.
-        val = bzrlib.xml8._encode_and_escape('foo bar')
+        val = bzrlib.xml_serializer.encode_and_escape('foo bar')
         self.assertEqual('foo bar"', val)
         # The second time should be cached
-        val2 = bzrlib.xml8._encode_and_escape('foo bar')
+        val2 = bzrlib.xml_serializer.encode_and_escape('foo bar')
         self.assertIs(val2, val)
 
     def test_ascii_with_xml(self):
         self.assertEqual('&amp;&apos;&quot;&lt;&gt;"',
-                         bzrlib.xml8._encode_and_escape('&\'"<>'))
+                         bzrlib.xml_serializer.encode_and_escape('&\'"<>'))
 
     def test_utf8_with_xml(self):
         # u'\xb5\xe5&\u062c'
         utf8_str = '\xc2\xb5\xc3\xa5&\xd8\xac'
         self.assertEqual('&#181;&#229;&amp;&#1580;"',
-                         bzrlib.xml8._encode_and_escape(utf8_str))
+                         bzrlib.xml_serializer.encode_and_escape(utf8_str))
 
     def test_unicode(self):
         uni_str = u'\xb5\xe5&\u062c'
         self.assertEqual('&#181;&#229;&amp;&#1580;"',
-                         bzrlib.xml8._encode_and_escape(uni_str))
+                         bzrlib.xml_serializer.encode_and_escape(uni_str))
+
+
+class TestMisc(TestCase):
+
+    def test_unescape_xml(self):
+        """We get some kind of error when malformed entities are passed"""
+        self.assertRaises(KeyError, bzrlib.xml8._unescape_xml, 'foo&bar;')
