@@ -25,6 +25,8 @@ NAT and other firewalls, so it's best to use it unless you explicitly want
 active, in which case aftp:// will be your friend.
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 import ftplib
 import getpass
@@ -40,6 +42,12 @@ from bzrlib import (
     osutils,
     urlutils,
     )
+from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
+    deprecated_in,
+    deprecated_passed,
+    warn,
+    )
 from bzrlib.trace import mutter, warning
 from bzrlib.transport import (
     AppendBasedFileStream,
@@ -90,7 +98,7 @@ class FtpTransport(ConnectedTransport):
         super(FtpTransport, self).__init__(base,
                                            _from_transport=_from_transport)
         self._unqualified_scheme = 'ftp'
-        if self._scheme == 'aftp':
+        if self._parsed_url.scheme == 'aftp':
             self.is_active = True
         else:
             self.is_active = False
@@ -164,6 +172,11 @@ class FtpTransport(ConnectedTransport):
         connection, credentials = self._create_connection(credentials)
         self._set_connection(connection, credentials)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _translate_ftp_error(self, err, path, extra=None,
                               unknown_exc=FtpPathError):
         """Try to translate an ftplib exception to a bzrlib exception.
@@ -191,6 +204,9 @@ class FtpTransport(ConnectedTransport):
             or 'file/directory not found' in s # filezilla server
             # Microsoft FTP-Service RNFR reply if file not found
             or (s.startswith('550 ') and 'unable to rename to' in extra)
+            # if containing directory doesn't exist, suggested by
+            # <https://bugs.launchpad.net/bzr/+bug/224373>
+            or (s.startswith('550 ') and "can't find folder" in s)
             ):
             raise errors.NoSuchFile(path, extra=extra)
         elif ('file exists' in s):
@@ -232,7 +248,7 @@ class FtpTransport(ConnectedTransport):
             mutter("FTP has not: %s: %s", abspath, e)
             return False
 
-    def get(self, relpath, decode=False, retries=0):
+    def get(self, relpath, decode=DEPRECATED_PARAMETER, retries=0):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
@@ -242,7 +258,10 @@ class FtpTransport(ConnectedTransport):
         We're meant to return a file-like object which bzr will
         then read from. For now we do this via the magic of StringIO
         """
-        # TODO: decode should be deprecated
+        if deprecated_passed(decode):
+            warn(deprecated_in((2,3,0)) %
+                 '"decode" parameter to FtpTransport.get()',
+                 DeprecationWarning, stacklevel=2)
         try:
             mutter("FTP get: %s", self._remote_path(relpath))
             f = self._get_FTP()
@@ -314,8 +333,9 @@ class FtpTransport(ConnectedTransport):
                     return len(bytes)
                 else:
                     return fp.counted_bytes
-            except (ftplib.error_temp,EOFError), e:
-                warning("Failure during ftp PUT. Deleting temporary file.")
+            except (ftplib.error_temp, EOFError), e:
+                warning("Failure during ftp PUT of %s: %s. Deleting temporary file."
+                    % (tmp_abspath, e, ))
                 try:
                     f.delete(tmp_abspath)
                 except:
@@ -328,8 +348,9 @@ class FtpTransport(ConnectedTransport):
                                        unknown_exc=errors.NoSuchFile)
         except ftplib.error_temp, e:
             if retries > _number_of_retries:
-                raise errors.TransportError("FTP temporary error during PUT %s. Aborting."
-                                     % self.abspath(relpath), orig_error=e)
+                raise errors.TransportError(
+                    "FTP temporary error during PUT %s: %s. Aborting."
+                    % (self.abspath(relpath), e), orig_error=e)
             else:
                 warning("FTP temporary error: %s. Retrying.", str(e))
                 self._reconnect()
@@ -350,7 +371,17 @@ class FtpTransport(ConnectedTransport):
         try:
             mutter("FTP mkd: %s", abspath)
             f = self._get_FTP()
-            f.mkd(abspath)
+            try:
+                f.mkd(abspath)
+            except ftplib.error_reply, e:
+                # <https://bugs.launchpad.net/bzr/+bug/224373> Microsoft FTP
+                # server returns "250 Directory created." which is kind of
+                # reasonable, 250 meaning "requested file action OK", but not what
+                # Python's ftplib expects.
+                if e[0][:3] == '250':
+                    pass
+                else:
+                    raise
             self._setmode(relpath, mode)
         except ftplib.error_perm, e:
             self._translate_ftp_error(e, abspath,
