@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,20 +18,22 @@
 """Text UI, write output to the console.
 """
 
-import codecs
-import getpass
+from __future__ import absolute_import
+
 import os
 import sys
 import time
-import warnings
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import codecs
+import getpass
+import warnings
+
 from bzrlib import (
     debug,
     progress,
     osutils,
-    symbol_versioning,
     trace,
     )
 
@@ -43,6 +45,116 @@ from bzrlib.ui import (
     )
 
 
+class _ChooseUI(object):
+
+    """ Helper class for choose implementation.
+    """
+
+    def __init__(self, ui, msg, choices, default):
+        self.ui = ui
+        self._setup_mode()
+        self._build_alternatives(msg, choices, default)
+
+    def _setup_mode(self):
+        """Setup input mode (line-based, char-based) and echo-back.
+
+        Line-based input is used if the BZR_TEXTUI_INPUT environment
+        variable is set to 'line-based', or if there is no controlling
+        terminal.
+        """
+        if os.environ.get('BZR_TEXTUI_INPUT') != 'line-based' and \
+           self.ui.stdin == sys.stdin and self.ui.stdin.isatty():
+            self.line_based = False
+            self.echo_back = True
+        else:
+            self.line_based = True
+            self.echo_back = not self.ui.stdin.isatty()
+
+    def _build_alternatives(self, msg, choices, default):
+        """Parse choices string.
+
+        Setup final prompt and the lists of choices and associated
+        shortcuts.
+        """
+        index = 0
+        help_list = []
+        self.alternatives = {}
+        choices = choices.split('\n')
+        if default is not None and default not in range(0, len(choices)):
+            raise ValueError("invalid default index")
+        for c in choices:
+            name = c.replace('&', '').lower()
+            choice = (name, index)
+            if name in self.alternatives:
+                raise ValueError("duplicated choice: %s" % name)
+            self.alternatives[name] = choice
+            shortcut = c.find('&')
+            if -1 != shortcut and (shortcut + 1) < len(c):
+                help = c[:shortcut]
+                help += '[' + c[shortcut + 1] + ']'
+                help += c[(shortcut + 2):]
+                shortcut = c[shortcut + 1]
+            else:
+                c = c.replace('&', '')
+                shortcut = c[0]
+                help = '[%s]%s' % (shortcut, c[1:])
+            shortcut = shortcut.lower()
+            if shortcut in self.alternatives:
+                raise ValueError("duplicated shortcut: %s" % shortcut)
+            self.alternatives[shortcut] = choice
+            # Add redirections for default.
+            if index == default:
+                self.alternatives[''] = choice
+                self.alternatives['\r'] = choice
+            help_list.append(help)
+            index += 1
+
+        self.prompt = u'%s (%s): ' % (msg, ', '.join(help_list))
+
+    def _getline(self):
+        line = self.ui.stdin.readline()
+        if '' == line:
+            raise EOFError
+        return line.strip()
+
+    def _getchar(self):
+        char = osutils.getchar()
+        if char == chr(3): # INTR
+            raise KeyboardInterrupt
+        if char == chr(4): # EOF (^d, C-d)
+            raise EOFError
+        return char
+
+    def interact(self):
+        """Keep asking the user until a valid choice is made.
+        """
+        if self.line_based:
+            getchoice = self._getline
+        else:
+            getchoice = self._getchar
+        iter = 0
+        while True:
+            iter += 1
+            if 1 == iter or self.line_based:
+                self.ui.prompt(self.prompt)
+            try:
+                choice = getchoice()
+            except EOFError:
+                self.ui.stderr.write('\n')
+                return None
+            except KeyboardInterrupt:
+                self.ui.stderr.write('\n')
+                raise KeyboardInterrupt
+            choice = choice.lower()
+            if choice not in self.alternatives:
+                # Not a valid choice, keep on asking.
+                continue
+            name, index = self.alternatives[choice]
+            if self.echo_back:
+                self.ui.stderr.write(name + '\n')
+            return index
+
+
 class TextUIFactory(UIFactory):
     """A UI factory for Text user interefaces."""
 
@@ -61,6 +173,27 @@ class TextUIFactory(UIFactory):
         # paints progress, network activity, etc
         self._progress_view = self.make_progress_view()
 
+    def choose(self, msg, choices, default=None):
+        """Prompt the user for a list of alternatives.
+
+        Support both line-based and char-based editing.
+
+        In line-based mode, both the shortcut and full choice name are valid
+        answers, e.g. for choose('prompt', '&yes\n&no'): 'y', ' Y ', ' yes',
+        'YES ' are all valid input lines for choosing 'yes'.
+
+        An empty line, when in line-based mode, or pressing enter in char-based
+        mode will select the default choice (if any).
+
+        Choice is echoed back if:
+        - input is char-based; which means a controlling terminal is available,
+          and osutils.getchar is used
+        - input is line-based, and no controlling terminal is available
+        """
+
+        choose_ui = _ChooseUI(self, msg, choices, default)
+        return choose_ui.interact()
+
     def be_quiet(self, state):
         if state and not self._quiet:
             self.clear_term()
@@ -78,18 +211,6 @@ class TextUIFactory(UIFactory):
         # to clear it.  We might need to separately check for the case of
         self._progress_view.clear()
 
-    def get_boolean(self, prompt):
-        while True:
-            self.prompt(prompt + "? [y/n]: ")
-            line = self.stdin.readline().lower()
-            if line in ('y\n', 'yes\n'):
-                return True
-            elif line in ('n\n', 'no\n'):
-                return False
-            elif line in ('', None):
-                # end-of-file; possibly should raise an error here instead
-                return None
-
     def get_integer(self, prompt):
         while True:
             self.prompt(prompt)
@@ -114,7 +235,7 @@ class TextUIFactory(UIFactory):
                 password = password[:-1]
         return password
 
-    def get_password(self, prompt='', **kwargs):
+    def get_password(self, prompt=u'', **kwargs):
         """Prompt the user for a password.
 
         :param prompt: The prompt to present the user
@@ -198,11 +319,14 @@ class TextUIFactory(UIFactory):
         :param kwargs: Dictionary of arguments to insert into the prompt,
             to allow UIs to reformat the prompt.
         """
+        if type(prompt) != unicode:
+            raise ValueError("prompt %r not a unicode string" % prompt)
         if kwargs:
             # See <https://launchpad.net/bugs/365891>
             prompt = prompt % kwargs
         prompt = prompt.encode(osutils.get_terminal_encoding(), 'replace')
         self.clear_term()
+        self.stdout.flush()
         self.stderr.write(prompt)
 
     def report_transport_activity(self, transport, byte_count, direction):
@@ -229,6 +353,9 @@ class TextUIFactory(UIFactory):
 
     def show_warning(self, msg):
         self.clear_term()
+        if isinstance(msg, unicode):
+            te = osutils.get_terminal_encoding()
+            msg = msg.encode(te, 'replace')
         self.stderr.write("bzr: warning: %s\n" % msg)
 
     def _progress_updated(self, task):
@@ -238,8 +365,12 @@ class TextUIFactory(UIFactory):
             warnings.warn("%r updated but no tasks are active" %
                 (task,))
         elif task != self._task_stack[-1]:
-            warnings.warn("%r is not the top progress task %r" %
-                (task, self._task_stack[-1]))
+            # We used to check it was the top task, but it's hard to always
+            # get this right and it's not necessarily useful: any actual
+            # problems will be evident in use
+            #warnings.warn("%r is not the top progress task %r" %
+            #     (task, self._task_stack[-1]))
+            pass
         self._progress_view.show_progress(task)
 
     def _progress_all_finished(self):
@@ -293,12 +424,18 @@ class TextProgressView(object):
         # correspond reliably to overall command progress
         self.enable_bar = False
 
+    def _avail_width(self):
+        # we need one extra space for terminals that wrap on last char
+        w = osutils.terminal_width() 
+        if w is None:
+            return None
+        else:
+            return w - 1
+
     def _show_line(self, s):
         # sys.stderr.write("progress %r\n" % s)
-        width = osutils.terminal_width()
+        width = self._avail_width()
         if width is not None:
-            # we need one extra space for terminals that wrap on last char
-            width = width - 1
             s = '%-*.*s' % (width, width, s)
         self._term_file.write('\r' + s + '\r')
 
@@ -341,6 +478,10 @@ class TextProgressView(object):
             return ''
 
     def _format_task(self, task):
+        """Format task-specific parts of progress bar.
+
+        :returns: (text_part, counter_part) both unicode strings.
+        """
         if not task.show_count:
             s = ''
         elif task.current_cnt is not None and task.total_cnt is not None:
@@ -356,21 +497,39 @@ class TextProgressView(object):
             t = t._parent_task
             if t.msg:
                 m = t.msg + ':' + m
-        return m + s
+        return m, s
 
     def _render_line(self):
         bar_string = self._render_bar()
         if self._last_task:
-            task_msg = self._format_task(self._last_task)
+            task_part, counter_part = self._format_task(self._last_task)
         else:
-            task_msg = ''
+            task_part = counter_part = ''
         if self._last_task and not self._last_task.show_transport_activity:
             trans = ''
         else:
             trans = self._last_transport_msg
-            if trans:
-                trans += ' | '
-        return (bar_string + trans + task_msg)
+        # the bar separates the transport activity from the message, so even
+        # if there's no bar or spinner, we must show something if both those
+        # fields are present
+        if (task_part or trans) and not bar_string:
+            bar_string = '| '
+        # preferentially truncate the task message if we don't have enough
+        # space
+        avail_width = self._avail_width()
+        if avail_width is not None:
+            # if terminal avail_width is unknown, don't truncate
+            current_len = len(bar_string) + len(trans) + len(task_part) + len(counter_part)
+            gap = current_len - avail_width
+            if gap > 0:
+                task_part = task_part[:-gap-2] + '..'
+        s = trans + bar_string + task_part + counter_part
+        if avail_width is not None:
+            if len(s) < avail_width:
+                s = s.ljust(avail_width)
+            elif len(s) > avail_width:
+                s = s[:avail_width]
+        return s
 
     def _repaint(self):
         s = self._render_line()
@@ -429,9 +588,11 @@ class TextProgressView(object):
         elif now >= (self._transport_update_time + 0.5):
             # guard against clock stepping backwards, and don't update too
             # often
-            rate = self._bytes_since_update / (now - self._transport_update_time)
-            msg = ("%6dKB %5dKB/s" %
-                    (self._total_byte_count>>10, int(rate)>>10,))
+            rate = (self._bytes_since_update
+                    / (now - self._transport_update_time))
+            # using base-10 units (see HACKING.txt).
+            msg = ("%6dkB %5dkB/s " %
+                    (self._total_byte_count / 1000, int(rate) / 1000,))
             self._transport_update_time = now
             self._last_repaint = now
             self._bytes_since_update = 0
@@ -447,16 +608,17 @@ class TextProgressView(object):
                 transfer_time = 0.001
             bps = self._total_byte_count / transfer_time
 
-        msg = ('Transferred: %.0fKiB'
-               ' (%.1fK/s r:%.0fK w:%.0fK'
-               % (self._total_byte_count / 1024.,
-                  bps / 1024.,
-                  self._bytes_by_direction['read'] / 1024.,
-                  self._bytes_by_direction['write'] / 1024.,
+        # using base-10 units (see HACKING.txt).
+        msg = ('Transferred: %.0fkB'
+               ' (%.1fkB/s r:%.0fkB w:%.0fkB'
+               % (self._total_byte_count / 1000.,
+                  bps / 1000.,
+                  self._bytes_by_direction['read'] / 1000.,
+                  self._bytes_by_direction['write'] / 1000.,
                  ))
         if self._bytes_by_direction['unknown'] > 0:
-            msg += ' u:%.0fK)' % (
-                self._bytes_by_direction['unknown'] / 1024.
+            msg += ' u:%.0fkB)' % (
+                self._bytes_by_direction['unknown'] / 1000.
                 )
         else:
             msg += ')'
